@@ -70,9 +70,10 @@ uniq_kernel_index()
 }
 
 template <typename Op, ::std::size_t CallNumber>
-using unique_kernel_name = oneapi::dpl::__par_backend_hetero::__unique_kernel_name<Op, CallNumber>;
+struct unique_kernel_name;
+
 template <typename Policy, int idx>
-using new_kernel_name = oneapi::dpl::__par_backend_hetero::__new_kernel_name<Policy, idx>;
+using new_kernel_name = unique_kernel_name<typename ::std::decay<Policy>::type, idx>;
 
 auto async_handler = [](sycl::exception_list ex_list) {
     for (auto& ex : ex_list)
@@ -333,7 +334,7 @@ get_host_pointer(Iter it)
     return &it.get_buffer().template get_access<mode>()[0] + temp_idx;
 }
 
-template <typename T, int Dim, sycl::access::mode AccMode, sycl::access::target AccTarget,
+template <typename T, int Dim, sycl::access::mode AccMode, __dpl_sycl::__target AccTarget,
           sycl::access::placeholder Placeholder>
 T*
 get_host_pointer(sycl::accessor<T, Dim, AccMode, AccTarget, Placeholder>& acc)
